@@ -7,13 +7,15 @@ moduleauthor:: J. Derek Tucker <jdtuck@sandia.gov>
 
 from scipy.interpolate import UnivariateSpline, interp1d
 from scipy.integrate import trapz, cumtrapz
-from scipy.linalg import norm, svd, cholesky, inv
+from scipy.linalg import norm, svd, cholesky, inv, pinv
 from scipy.stats.mstats import mquantiles
+from scipy import special as sp
 from numpy import zeros, interp, finfo, double, sqrt, diff, linspace
 from numpy import arccos, sin, cos, arange, ascontiguousarray, round
 from numpy import ones, real, pi, cumsum, fabs, cov, diagflat, inner
 from numpy import gradient, column_stack, append, mean, hstack, median
 from numpy import insert, vectorize, ceil, mod, array, quantile, dot
+from numpy import exp
 from joblib import Parallel, delayed
 import numpy.random as rn
 import optimum_reparamN2 as orN2
@@ -38,7 +40,7 @@ def smooth_data(f, sparam):
     M = f.shape[0]
     N = f.shape[1]
 
-    fo = zeros((M,N))
+    fo = f.copy()
     for k in range(1, sparam):
         for r in range(0, N):
             fo[1:(M - 2), r] = (f[0:(M - 3), r] + 2 * f[1:(M - 2), r] +
@@ -292,8 +294,8 @@ def elastic_distance(f1, f2, time, method="DP2", lam=0.0):
     q2 = f_to_srsf(f2, time)
 
     gam = optimum_reparam(q1, time, q2, method, lam)
-    fw = interp((time[-1] - time[0]) * gam + time[0], time, f2)
-    qw = f_to_srsf(fw, time)
+    fw = warp_f_gamma(time, f2, gam)
+    qw = warp_q_gamma(time, q2, gam)
 
     Dy = sqrt(trapz((qw - q1) ** 2, time))
     M = time.shape[0]
@@ -762,15 +764,15 @@ def geigen(Amat, Bmat, Cmat):
     Cfacinv = inv(Cfac)
     Dmat = Bfacinvt.dot(Amat).dot(Cfacinv)
     if p >= q:
-        u, d, v = svd(Dmat)
+        u, d, vh = svd(Dmat)
         values = d
         Lmat = Bfacinv.dot(u)
-        Mmat = Cfacinv.dot(v.transpose())
+        Mmat = Cfacinv.dot(vh)
     else:
-        u, d, v = svd(Dmat.transpose())
+        u, d, vh = svd(Dmat.transpose())
         values = d
         Lmat = Bfacinv.dot(u)
-        Mmat = Cfacinv.dot(v.transpose())
+        Mmat = Cfacinv.dot(vh)
 
     return values, Lmat, Mmat
 
@@ -930,6 +932,22 @@ def resamplefunction(x, n):
     xn = interp(arange(0, n)/double(n-1), arange(0, T)/double(T-1), x)
     return(xn)
 
+
+def basis_fourierd(f_domain, numBasis):
+    result = zeros((f_domain.shape[0], 2*numBasis))
+    for i in range(0,2*numBasis):
+        j = ceil(i/2)
+        if mod(i,2) == 1:
+            result[:,i] = 1/sqrt(pi) * sin(2*j*pi*f_domain)
+
+        if mod(i,2) == 0:
+            result[:,i] = 1/sqrt(pi) * cos(2*j*pi*f_domain)
+
+    out = {"x":f_domain, "matrix":result}
+
+    return(out)
+
+
 def basis_fourier(f_domain, numBasis, fourier_p):
     result = zeros((f_domain.shape[0], 2*numBasis))
     for i in range(0,2*numBasis):
@@ -944,16 +962,49 @@ def basis_fourier(f_domain, numBasis, fourier_p):
 
     return(out)
 
+
+def legendre(N,X) :
+    matrixReturn = zeros((N+1,X.shape[0]))
+    for i in enumerate(X) :
+        currValues = sp.lpmn(N,N,i[1])
+        matrixReturn[:,i[0]] = array([j[N] for j in currValues[0]])
+    return matrixReturn
+
+
+def basis_legendre(f_domain, numBasis, fourier_p):
+    result = zeros((f_domain.shape[0], 2*numBasis))
+    for i in range(0,2*numBasis):
+        f_domain_scaled = 2*(f_domain/fourier_p) - 1
+        tmp = legendre(i+1, f_domain_scaled)
+        result[:, i] = tmp[0, :]
+
+    out = {"x":f_domain, "matrix":result}
+
+    return(out)
+
+
+def exp2corr(sigma2, phi, ds):
+    out = sigma2 * exp(-ds**2/(2*phi**2))
+    return(out)
+
+
+def exp2corr2(phi, ds):
+    out = exp(-ds**2/(2*phi**2))
+    return(out)
+
+
 def statsFun(vec):
     a = quantile(vec,0.025,axis=1)
     b = quantile(vec,0.975,axis=1)
     out = column_stack((a, b))
     return(out)
 
+
 def f_exp1(g):
     out = bay.bcalcY(f_L2norm(g), g)
     return(out)
 
+
 def f_L2norm(f):
     x = linspace(0,1,f.shape[0])
     out = bay.border_l2norm(x,f)
@@ -998,3 +1049,8 @@ def f_phiinv(psi):
 def norm_gam(gam):
     gam = (gam-gam[0])/(gam[-1]-gam[0])
     return(gam)
+
+
+def mrdivide(a, b):
+    c = dot(a, pinv(b))
+    return(c)
